@@ -3,4 +3,4 @@
 #define HAVE_ASSERT_H 1
 #define HAVE_UNISTD_H 1
 #define HAVE_MATH_H 1
-#define MPI 1
+#define MSG_COMMS_MPI 1
