@@ -22,6 +22,7 @@ extern void          GA_Add_constant_patch(int g,int *lo,int *hi,void *alpha);
 extern void          GA_Add_diagonal(int g_a, int g_v);
 extern void          GA_Add(void *alpha, int g_a, void* beta, int g_b, int g_c); 
 extern int           GA_Allocate(int g_a);
+extern int           GA_Deallocate(int g_a);
 extern int           GA_Assemble_duplicate(int g_a, char *name, void *ptr);
 extern void          GA_Brdcst(void *buf, int lenbuf, int root);
 extern SingleComplex GA_Cdot(int g_a, int g_b); 
@@ -37,6 +38,7 @@ extern int           GA_Compare_distr(int g_a, int g_b);
 extern void          GA_Copy(int g_a, int g_b); 
 extern int           GA_Create_handle(void);
 extern int           GA_Create_mutexes(int number);
+extern int           GA_Deallocate(int g_a);
 extern double        GA_Ddot(int g_a, int g_b); 
 extern void          GA_Destroy(int g_a);
 extern int           GA_Destroy_mutexes(void);
@@ -104,12 +106,14 @@ extern int           GA_Nnodes(void);
 extern int           GA_Nodeid(void);
 extern void          GA_Norm1(int g_a, double *nm);
 extern void          GA_Norm_infinity(int g_a, double *nm);
+extern int           GA_Overlay(int g_a, int g_p);
 extern int           GA_Pgroup_absolute_id(int pgroup, int pid);
 extern void          GA_Pgroup_brdcst(int grp, void *buf, int lenbuf, int root);
 extern void          GA_Pgroup_cgop(int grp, SingleComplex x[], int n, char *op);
 extern int           GA_Pgroup_create(int *list, int count);
 extern int           GA_Pgroup_destroy(int grp);
 extern void          GA_Pgroup_dgop(int grp, double x[], int n, char *op);
+extern int           GA_Pgroup_duplicate(int grp);
 extern void          GA_Pgroup_fgop(int grp, float x[], int n, char *op);   
 extern int           GA_Pgroup_get_default(void);
 extern int           GA_Pgroup_get_mirror(void);
@@ -119,6 +123,7 @@ extern void          GA_Pgroup_lgop(int grp, long x[], int n, char *op);
 extern void          GA_Pgroup_llgop(int grp, long long x[], int n, char *op);
 extern int           GA_Pgroup_nnodes(int grp_id);
 extern int           GA_Pgroup_nodeid(int grp_id);
+extern int           GA_Pgroup_self();
 extern void          GA_Pgroup_set_default(int p_handle);
 extern int           GA_Pgroup_split(int grp_id, int num_group);
 extern int           GA_Pgroup_split_irreg(int grp_id, int color);
@@ -142,6 +147,7 @@ extern void          GA_Set_array_name(int g_a, char *name);
 extern void          GA_Set_block_cyclic(int g_a, int dims[]);
 extern void          GA_Set_block_cyclic_proc_grid(int g_a, int block[], int proc_grid[]);
 extern void          GA_Set_tiled_proc_grid(int g_a, int block[], int proc_grid[]);
+extern void          GA_Set_tiled_irreg_proc_grid(int g_a, int mapc[], int nblocks[], int proc_grid[]);
 extern void          GA_Set_chunk(int g_a, int chunk[]);
 extern void          GA_Set_data(int g_a, int ndim, int dims[], int type);
 extern void          GA_Set_debug(int flag);
@@ -151,6 +157,7 @@ extern void          GA_Set_ghosts(int g_a, int width[]);
 extern void          GA_Set_irreg_distr(int g_a, int map[], int block[]);
 extern void          GA_Set_irreg_flag(int g_a, int flag);
 extern void          GA_Set_memory_limit(size_t limit);
+extern void          GA_Set_memory_dev(int g_a, char *device);
 extern void          GA_Set_pgroup(int g_a, int p_handle);
 extern void          GA_Set_restricted(int g_a, int list[], int size);
 extern void          GA_Set_restricted_range(int g_a, int lo_proc, int hi_proc);
@@ -192,6 +199,7 @@ extern void          NGA_Access(int g_a, int lo[], int hi[], void *ptr, int ld[]
 extern void          NGA_Acc(int g_a, int lo[], int hi[],void* buf,int ld[],void* alpha);
 extern void          NGA_Add_patch(void * alpha, int g_a, int alo[], int ahi[], void * beta,  int g_b, int blo[], int bhi[], int g_c, int clo[], int chi[]);
 extern int           NGA_Allocate(int g_a);
+extern int           NGA_Deallocate(int g_a);  
 extern void          NGA_Alloc_gatscat_buf(int nelems);
 extern SingleComplex NGA_Cdot_patch(int g_a, char t_a, int alo[], int ahi[], int g_b, char t_b, int blo[], int bhi[]);
 extern int           NGA_Compare_distr(int g_a, int g_b); 
@@ -206,6 +214,7 @@ extern int           NGA_Create(int type,int ndim,int dims[], char *name, int ch
 extern int           NGA_Create_irreg_config(int type,int ndim,int dims[],char *name, int block[], int map[], int p_handle);
 extern int           NGA_Create_irreg(int type,int ndim,int dims[],char *name, int block[], int map[]);
 extern int           NGA_Create_handle(void);
+extern int           NGA_Deallocate(int g_a); 
 extern int           NGA_Deregister_type(int type);
 extern void          NGA_Destroy(int g_a);
 extern int           NGA_Destroy_mutexes(void);
@@ -216,7 +225,7 @@ extern int           NGA_Duplicate(int g_a, char* array_name);
 extern void          NGA_Error(char *str, int code);
 extern float         NGA_Fdot_patch(int g_a, char t_a, int alo[], int ahi[], int g_b, char t_b, int blo[], int bhi[]);
 extern void          NGA_Fence(void);
-extern void          NGA_Free_gatscat_buf(void);
+extern void          NGA_Free_gatscat_buf();
 extern void          NGA_Fill(int g_a, void *value);
 extern void          NGA_Fill_patch(int g_a, int lo[], int hi[], void *val);
 extern void          NGA_Gather(int g_a, void *v, int* subsArray[], int n);
@@ -271,18 +280,27 @@ extern int           NGA_Nodeid(void);
 extern void          NGA_Periodic_acc(int g_a, int lo[], int hi[],void* buf,int ld[],void* alpha);
 extern void          NGA_Periodic_get(int g_a, int lo[], int hi[], void* buf, int ld[]); 
 extern void          NGA_Periodic_put(int g_a, int lo[], int hi[], void* buf, int ld[]); 
+extern int           NGA_Overlay(int g_a, int g_p);
 extern int           NGA_Pgroup_absolute_id(int pgroup, int pid);
 extern int           NGA_Pgroup_create(int *list, int count);
 extern int           NGA_Pgroup_destroy(int grp);
+extern void          NGA_Pgroup_dgop(int grp, double x[], int n, char *op);
+extern int           NGA_Pgroup_duplicate(int grp);
+extern void          NGA_Pgroup_fgop(int grp, float x[], int n, char *op);   
 extern int           NGA_Pgroup_get_default(void);
 extern int           NGA_Pgroup_get_mirror(void);
 extern int           NGA_Pgroup_get_world(void);
+extern void          NGA_Pgroup_igop(int grp, int x[], int n, char *op);
+extern void          NGA_Pgroup_lgop(int grp, long x[], int n, char *op);
+extern void          NGA_Pgroup_llgop(int grp, long long x[], int n, char *op);
 extern int           NGA_Pgroup_nnodes(int grp_id);
 extern int           NGA_Pgroup_nodeid(int grp_id);
+extern int           NGA_Pgroup_self();
 extern void          NGA_Pgroup_set_default(int p_handle);
 extern int           NGA_Pgroup_split(int grp_id, int num_group);
 extern int           NGA_Pgroup_split_irreg(int grp_id, int color);
 extern void          NGA_Pgroup_sync(int grp_id);
+extern void          NGA_Pgroup_zgop(int grp, DoubleComplex x[], int n, char *op);
 extern void          NGA_Print_patch(int g_a, int lo[], int hi[], int pretty);
 extern void          NGA_Proc_topology(int g_a, int proc, int coord[]);
 extern void          NGA_Put(int g_a, int lo[], int hi[], void* buf, int ld[]); 
@@ -312,6 +330,7 @@ extern void          NGA_Set_array_name(int g_a, char *name);
 extern void          NGA_Set_block_cyclic(int g_a, int dims[]);
 extern void          NGA_Set_block_cyclic_proc_grid(int g_a, int block[], int proc_grid[]);
 extern void          NGA_Set_tiled_proc_grid(int g_a, int block[], int proc_grid[]);
+extern void          NGA_Set_tiled_irreg_proc_grid(int g_a, int mapc[], int nblocks[], int proc_grid[]);
 extern void          NGA_Set_chunk(int g_a, int chunk[]);
 extern void          NGA_Set_data(int g_a, int ndim, int dims[], int type);
 extern void          NGA_Set_debug(int flag);
@@ -319,11 +338,11 @@ extern void          NGA_Set_ghosts(int g_a, int width[]);
 extern void          NGA_Set_irreg_distr(int g_a, int map[], int block[]);
 extern void          NGA_Set_irreg_flag(int g_a, int flag);
 extern void          NGA_Set_memory_limit(size_t limit);
+extern void          NGA_Set_memory_dev(int g_a, char *device);
 extern void          NGA_Set_pgroup(int g_a, int p_handle);
+extern void          NGA_Set_property(int g_a, char *property);
 extern void          NGA_Set_restricted(int g_a, int list[], int size);
 extern void          NGA_Set_restricted_range(int g_a, int lo_proc, int hi_proc);
-extern void          NGA_Set_property(int g_a, char *property);
-extern void          NGA_Unset_property(int g_a);
 extern void          NGA_Strided_acc(int g_a, int lo[], int hi[], int skip[], void* buf, int ld[], void *alpha); 
 extern void          NGA_Strided_get(int g_a, int lo[], int hi[], int skip[], void* buf, int ld[]); 
 extern void          NGA_Strided_put(int g_a, int lo[], int hi[], int skip[], void* buf, int ld[]); 
@@ -331,6 +350,7 @@ extern void          NGA_Sync(void);
 extern void          NGA_Terminate(void);
 extern int           NGA_Total_blocks(int g_a);   
 extern void          NGA_Unlock(int mutex);
+extern void          NGA_Unset_property(int g_a);
 extern void          NGA_Update_ghosts(int g_a);
 extern int           NGA_Update_ghost_dir(int g_a, int dimension, int idir, int flag);
 extern void          NGA_Update_ghosts_nb(int g_a, ga_nbhdl_t *nbhandle);
@@ -362,10 +382,14 @@ extern void          GA_Patch_enum(int g_a, int lo, int hi, void *start, void *i
 extern void          GA_Recip_patch64(int g_a,int64_t *lo, int64_t *hi);
 extern void          GA_Scan_add64(int g_a, int g_b, int g_sbit, int64_t lo, int64_t hi, int excl);
 extern void          GA_Scan_copy64(int g_a, int g_b, int g_sbit, int64_t lo, int64_t hi);
+extern void          GA_Set_block_cyclic64(int g_a, int64_t dims[]);
+extern void          GA_Set_block_cyclic_proc_grid64(int g_a, int64_t block[], int64_t proc_grid[]);
 extern void          GA_Set_chunk64(int g_a, int64_t chunk[]);
 extern void          GA_Set_data64(int g_a, int ndim, int64_t dims[], int type);
 extern void          GA_Set_ghosts64(int g_a, int64_t width[]);
 extern void          GA_Set_irreg_distr64(int g_a, int64_t map[], int64_t block[]);
+extern void          GA_Set_tiled_proc_grid64(int g_a, int64_t block[], int64_t proc_grid[]);
+extern void          GA_Set_tiled_irreg_proc_grid64(int g_a, int64_t mapc[], int64_t nblocks[], int64_t proc_grid[]);
 extern void          GA_Sgemm64(char ta, char tb, int64_t m, int64_t n, int64_t k, float alpha, int g_a, int g_b, float beta,  int g_c );
 extern void          GA_Step_bound_info_patch64(int g_xx, int64_t xxlo[], int64_t xxhi[], int g_vv, int64_t vvlo[], int64_t vvhi[], int g_xxll, int64_t xxlllo[], int64_t xxllhi[], int64_t g_xxuu,  int64_t xxuulo[], int64_t xxuuhi[], void *boundmin, void *wolfemin, void *boundmax);
 extern void          GA_Step_max_patch64(int g_a, int64_t alo[], int64_t  ahi[], int g_b, int64_t blo[], int64_t  bhi[], void *step);
@@ -427,14 +451,22 @@ extern void          NGA_Scatter_flat64(int g_a, void *v, int64_t subsArray[], i
 extern void          NGA_Scatter_acc64(int g_a, void *v, int64_t* subsArray[], int64_t n, void *alpha);
 extern void          NGA_Scatter_acc_flat64(int g_a, void *v, int64_t subsArray[], int64_t n, void *alpha);
 extern void          NGA_Select_elem64(int g_a, char* op, void* val, int64_t* index);
+extern void          NGA_Set_block_cyclic64(int g_a, int64_t dims[]);
+extern void          NGA_Set_block_cyclic_proc_grid64(int g_a, int64_t block[], int64_t proc_grid[]);
 extern void          NGA_Set_data64(int g_a, int ndim, int64_t dims[], int type);
 extern void          NGA_Set_ghosts64(int g_a, int64_t width[]);
 extern void          NGA_Set_irreg_distr64(int g_a, int64_t map[], int64_t block[]);
+extern void          NGA_Set_tiled_proc_grid64(int g_a, int64_t block[], int64_t proc_grid[]);
+extern void          NGA_Set_tiled_irreg_proc_grid64(int g_a, int64_t mapc[], int64_t nblocks[], int64_t proc_grid[]);
 extern void          NGA_Strided_acc64(int g_a, int64_t lo[], int64_t hi[], int64_t skip[], void* buf, int64_t  ld[], void *alpha); 
 extern void          NGA_Strided_get64(int g_a, int64_t lo[], int64_t hi[], int64_t skip[], void* buf, int64_t  ld[]); 
 extern void          NGA_Strided_put64(int g_a, int64_t lo[], int64_t hi[], int64_t skip[], void* buf, int64_t  ld[]); 
 extern DoubleComplex NGA_Zdot_patch64(int g_a, char t_a, int64_t alo[], int64_t ahi[], int g_b, char t_b, int64_t blo[], int64_t bhi[]);
 extern void          NGA_Zero_patch64(int g_a, int64_t lo[], int64_t hi[]);
+extern int           NGA_Sprs_array_create64(int64_t idim, int64_t jdim, int type);
+extern void          NGA_Sprs_array_add_element64(int s_a, int64_t idx, int64_t jdx, void *val);
+extern void          NGA_Sprs_array_row_distribution64(int s_a, int iproc, int64_t *lo, int64_t *hi);
+extern void          NGA_Sprs_array_column_distribution64(int s_a, int iproc, int64_t *lo, int64_t *hi);
 
 #ifdef __cplusplus
 }
