@@ -1,12 +1,13 @@
-#! /bin/sh
+#! /bin/bash
 
 # Exit on error
 set -ev
-
 os=`uname`
 TRAVIS_ROOT="$1"
 PORT="$2"
 MPI_IMPL="$3"
+USE_CMAKE="$4"
+FORTRAN_COMPILER="$5"
 
 # Environment variables
 export CFLAGS="-std=c99"
@@ -20,15 +21,7 @@ MAKE_JNUM=4
 # Capture details of build
 case "$MPI_IMPL" in
     mpich)
-        case "$os" in
-            Darwin)
-                echo "Mac"
-            ;;
-            Linux)
-                echo "Linux"
-                export PATH=$TRAVIS_ROOT/mpich/bin:$PATH
-            ;;
-        esac
+        export PATH=$TRAVIS_ROOT/mpich/bin:$PATH
         mpichversion
         mpicc -show
         ;;
@@ -49,34 +42,92 @@ case "$MPI_IMPL" in
         #ompi_info --arch --config
         mpicc --showme:command
         ;;
+    intel)
+	source /opt/intel/oneapi/setvars.sh --force || true
+	;;
 esac
 
 # Configure and build
-./autogen.sh $TRAVIS_ROOT
+if [ "$USE_CMAKE" = "Y" ] ; then
+    echo 'nothing to do here for cmake '
+else
+    ./autogen.sh $TRAVIS_ROOT
+fi
+case "$os" in
+    Darwin)
+        echo "Mac CFLAGS" $CFLAGS
+        ;;
+    Linux)
+	export CFLAGS="${CFLAGS} -fPIC "
+        echo "Linux CFLAGS" $CFLAGS
+        ;;
+esac
+if [ "$USE_CMAKE" = "Y" ] ; then
+case "x$PORT" in
+    xmpi-ts)
+        ga_rt="MPI_2SIDED"
+        ;;
+    xmpi-pr)
+        ga_rt="MPI_PROGRESS_RANK"
+        ;;
+    xmpi-pt)
+        ga_rt="MPI_PROGRESS_THREAD"
+        ;;
+    xmpi-mt)
+        ga_rt="MPI_MULTITHREADED"
+        ;;
+    x)
+        ga_rt="MPI_2SIDED"
+        ;;
+    x*)
+	echo PORT = "$PORT" not recognized
+	exit 1
+        ;;
+esac
+    mkdir -p build
+    cd build
+    echo FORTRAN_COMPILER is $FORTRAN_COMPILER
+    mpif90 -show || true
+    FC="$FORTRAN_COMPILER" cmake -DCMAKE_Fortran_COMPILER="$FORTRAN_COMPILER"  -DMPIEXEC_MAX_NUMPROCS=5 -DGA_RUNTIME="$ga_rt" ../
+else
 case "x$PORT" in
     xofi)
         ./configure --with-ofi=$TRAVIS_ROOT/libfabric
-        if [[ "$os" == "Darwin" ]]; then
+        if [[ "$os" = "Darwin" ]] ; then
             export COMEX_OFI_LIBRARY=$TRAVIS_ROOT/libfabric/lib/libfabric.dylib
         fi
         ;;
     xarmci)
-        ./configure --with-armci=$TRAVIS_ROOT/external-armci CFLAGS=-pthread LIBS=-lpthread
+        ./configure --with-armci=$TRAVIS_ROOT/external-armci CFLAGS="${CFLAGS} -pthread " LIBS=-lpthread
         ;;
     x)
-        ./configure $CONFIG_OPTS
+        ./configure ${CONFIG_OPTS}
+        ;;
+    xmpi-pr)
+        if [[ "$USE_SICM" = "Y" ]] ; then
+            export CFLAGS="-DUSE_SICM=1 -I${HOME}/no_cache/SICM/include -I${HOME}/no_cache/SICM/include/public ${CFLAGS}"
+            export LDFLAGS="-L${HOME}/no_cache/jemalloc/lib -ljemalloc -L${HOME}/no_cache/SICM/lib -lsicm ${LDFLAGS}"
+            export LD_LIBRARY_PATH="${HOME}/no_cache/SICM/lib:${HOME}/no_cache/jemalloc/lib:${LD_LIBRARY_PATH}"
+        fi
+        ./configure --with-${PORT} ${CONFIG_OPTS}
         ;;
     x*)
-        ./configure --with-${PORT} $CONFIG_OPTS
+        ./configure --with-${PORT} ${CONFIG_OPTS}
         ;;
 esac
+fi
 
 # build libga
 make V=0 -j ${MAKE_JNUM}
 
 # build test programs
-make V=0 checkprogs -j ${MAKE_JNUM}
-
+if [ "$USE_CMAKE" = "Y" ] ; then
+    cd global/testing
+    make
+    cd ../..
+else
+    make V=0 checkprogs -j ${MAKE_JNUM}
+fi
 # run one test
 MAYBE_OVERSUBSCRIBE=
 if test "x$os" = "xDarwin" && test "x$MPI_IMPL" = "xopenmpi"
@@ -106,4 +157,3 @@ then
 else
     mpirun -n 4 ${MAYBE_OVERSUBSCRIBE} ${TEST_NAME}
 fi
-
