@@ -31,7 +31,7 @@ MAKE_JNUM=4
 # we need m4 at least version 1.4.16
 M4_VERSION=1.4.17
 LIBTOOL_VERSION=2.4.6
-AUTOCONF_VERSION=2.69
+AUTOCONF_VERSION=2.71
 AUTOMAKE_VERSION=1.11.6
 
 # check whether we can reach ftp.gnu.org
@@ -109,7 +109,7 @@ else
     if [ -f secure_snprintf.patch ] ; then
         echo secure_snprintf.patch already exists! Using existing copy.
     else
-        ${download} secure_snprintf.patch https://raw.githubusercontent.com/macports/macports-ports/master/devel/m4/files/secure_snprintf.patch
+	${download} secure_snprintf.patch https://raw.githubusercontent.com/macports/macports-ports/edf0ee1e2cf/devel/m4/files/secure_snprintf.patch
     fi
     if patch -p0 -N < secure_snprintf.patch ; then
         echo patch applied
@@ -163,6 +163,14 @@ else
     cd ${TOP}/${TDIR}
     cp ${TOP}/bin/config.guess ./build-aux/config.guess
     cp ${TOP}/bin/config.sub ./build-aux/config.sub
+# patch for ifx -loopopt=0 issue
+    # patch for ifort libclang_rt.osx.a https://github.com/nwchemgit/nwchem/issues/171
+    if [ -f  ${TOP}/../travis/ifort_ldflags.patch ] ; then
+	cp ${TOP}/../travis/ifort_ldflags.patch .
+    else
+	${download} ifort_ldflags.patch https://raw.githubusercontent.com/GlobalArrays/ga/master/travis/ifort_ldflags.patch
+    fi
+    patch -p1 <  ifort_ldflags.patch
     ./configure --prefix=${TOP} && make -j ${MAKE_JNUM} && make install
     if [ "x$?" != "x0" ] ; then
         echo FAILURE 3
